@@ -30,6 +30,7 @@
 
 #include <json_writer.h>
 #include <SNAPSHOT.h>
+#include "utils.h"
 
 int dump_zeros;
 int reset_history;
@@ -95,7 +96,7 @@ static int useless_number(const char *id)
 {
 	int i;
 
-	for (i = 0; i < sizeof(useless_numbers)/sizeof(*useless_numbers); i++)
+	for (i = 0; i < ARRAY_SIZE(useless_numbers); i++)
 		if (strcmp(id, useless_numbers[i]) == 0)
 			return 1;
 	return 0;
@@ -284,6 +285,7 @@ static void dump_kern_db(FILE *fp, int to_hist)
 
 	h = hist_db;
 	if (jw) {
+		jsonw_start_object(jw);
 		jsonw_pretty(jw, pretty);
 		jsonw_name(jw, info_source);
 		jsonw_start_object(jw);
@@ -316,6 +318,8 @@ static void dump_kern_db(FILE *fp, int to_hist)
 	}
 
 	if (jw) {
+		jsonw_end_object(jw);
+
 		jsonw_end_object(jw);
 		jsonw_destroy(&jw);
 	}
@@ -328,6 +332,7 @@ static void dump_incr_db(FILE *fp)
 
 	h = hist_db;
 	if (jw) {
+		jsonw_start_object(jw);
 		jsonw_pretty(jw, pretty);
 		jsonw_name(jw, info_source);
 		jsonw_start_object(jw);
@@ -363,6 +368,8 @@ static void dump_incr_db(FILE *fp)
 	}
 
 	if (jw) {
+		jsonw_end_object(jw);
+
 		jsonw_end_object(jw);
 		jsonw_destroy(&jw);
 	}
